use std::cmp::max;
use std::fs::File;
use std::io::{BufRead, BufReader};

fn main() {
    println!("{}", find_max_total(get_triangle()));
}

fn get_triangle() -> Vec<Vec<usize>> {
    let file = File::open("triangle.txt").expect("Could not read file");
    let reader = BufReader::new(file);
    let mut grid = Vec::new();
    for line in reader.lines() {
        let line = line.expect("Could not get line");
        let nums = line
            .split_whitespace()
            .map(|num| num.parse::<usize>().expect("Could not parse"))
            .collect();
        grid.push(nums);
    }
    grid
}

fn find_max_total(grid: Vec<Vec<usize>>) -> usize {
    let triangle_height = grid.len();
    let mut idx = triangle_height - 1;
    let mut highest_sums = match grid.get(idx) {
        Some(sums) => sums.to_vec(),
        None => panic!("Impossible path: check grid"),
    };
    while idx > 0 {
        idx -= 1;
        let line = grid.get(idx).unwrap();
        let mut new_highest_sums = Vec::new();
        for (pos, &num) in line.iter().enumerate() {
            new_highest_sums.push(max(num + highest_sums[pos], num + highest_sums[pos + 1]))
        }
        highest_sums = new_highest_sums;
    }
    highest_sums[0]
}

#[cfg(test)]
mod test_18 {
    use crate::find_max_total;

    #[test]
    fn test_find_max_total() {
        let grid = vec![vec![3], vec![7, 4], vec![2, 4, 6], vec![8, 5, 9, 3]];
        assert_eq!(23, find_max_total(grid));
    }
}
